@@ -4,8 +4,13 @@ package e2e
 
 import (
 	"fmt"
+	"net"
+	"reflect"
 	"strings"
 	"testing"
+	"time"
+
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/version"
 
 	"github.com/blang/semver"
 	"github.com/stretchr/testify/require"
@@ -490,7 +495,7 @@ func TestGrpcAddressCatalogSource(t *testing.T) {
 		},
 		Spec: v1alpha1.CatalogSourceSpec{
 			SourceType: v1alpha1.SourceTypeGrpc,
-			Address:    fmt.Sprintf("%s:%s", mainCopy.Status.PodIP, "50051"),
+			Address:    net.JoinHostPort(mainCopy.Status.PodIP, "50051"),
 		},
 	}
 
@@ -521,7 +526,7 @@ func TestGrpcAddressCatalogSource(t *testing.T) {
 	// Update the catalog's address to point at the other registry pod's cluster ip
 	addressSource, err = crc.OperatorsV1alpha1().CatalogSources(testNamespace).Get(addressSourceName, metav1.GetOptions{})
 	require.NoError(t, err)
-	addressSource.Spec.Address = fmt.Sprintf("%s:%s", replacementCopy.Status.PodIP, "50051")
+	addressSource.Spec.Address = net.JoinHostPort(replacementCopy.Status.PodIP, "50051")
 	_, err = crc.OperatorsV1alpha1().CatalogSources(testNamespace).Update(addressSource)
 	require.NoError(t, err)
 
@@ -694,6 +699,249 @@ func TestDeleteGRPCRegistryPodTriggersRecreation(t *testing.T) {
 	require.Equal(t, 1, len(registryPods.Items), "unexpected number of replacement registry pods found")
 }
 
+const (
+	openshiftregistryFQDN = "image-registry.openshift-image-registry.svc:5000/openshift-operators"
+	catsrcImage           = "docker://quay.io/olmtest/catsrc-update-test:"
+)
+
+func TestCatalogImageUpdate(t *testing.T) {
+	// Create an image based catalog source from public Quay image
+	// Use a unique tag as identifier
+	// See https://quay.io/repository/olmtest/catsrc-update-test?namespace=olmtest for registry
+	// Push an updated version of the image with the same identifier
+	// Confirm catalog source polling feature is working as expected: a newer version of the catalog source pod comes up
+	// etcd operator updated from 0.9.0 to 0.9.2-clusterwide
+	// Subscription should detect the latest version of the operator in the new catalog source and pull it
+
+	// create internal registry for purposes of pushing/pulling IF running e2e test locally
+	// registry is insecure and for purposes of this test only
+	c := newKubeClient(t)
+	crc := newCRClient(t)
+
+	local, err := Local(c)
+	if err != nil {
+		t.Fatalf("cannot determine if test running locally or on CI: %s", err)
+	}
+
+	var registryURL string
+	var registryAuth string
+	if local {
+		registryURL, err = createDockerRegistry(c, testNamespace)
+		if err != nil {
+			t.Fatalf("error creating container registry: %s", err)
+		}
+		defer deleteDockerRegistry(c, testNamespace)
+
+		// ensure registry pod is ready before attempting port-forwarding
+		_ = awaitPod(t, c, testNamespace, registryName, podReady)
+		err = registryPortForward(testNamespace)
+		if err != nil {
+			t.Fatalf("port-forwarding local registry: %s", err)
+		}
+	} else {
+		registryURL = openshiftregistryFQDN
+		registryAuth, err = openshiftRegistryAuth(c, testNamespace)
+		if err != nil {
+			t.Fatalf("error getting openshift registry authentication: %s", err)
+		}
+	}
+
+	// testImage is the name of the image used throughout the test - the image overwritten by skopeo
+	// the tag is generated randomly and appended to the end of the testImage
+	testImage := fmt.Sprint("docker://", registryURL, "/catsrc-update", ":")
+	tag := genName("x")
+
+	// 1. copy old catalog image into test-specific tag in internal docker registry
+	// create skopeo pod to actually do the work of copying (on openshift) or exec out to local skopeo
+	if local {
+		_, err := skopeoLocalCopy(testImage, tag, catsrcImage, "old")
+		if err != nil {
+			t.Fatalf("error copying old registry file: %s", err)
+		}
+	} else {
+		skopeoArgs := skopeoCopyCmd(testImage, tag, catsrcImage, "old", registryAuth)
+		err = createSkopeoPod(c, skopeoArgs, testNamespace)
+		if err != nil {
+			t.Fatalf("error creating skopeo pod: %s", err)
+		}
+
+		// wait for skopeo pod to exit successfully
+		awaitPod(t, c, testNamespace, skopeo, func(pod *corev1.Pod) bool {
+			return pod.Status.Phase == corev1.PodSucceeded
+		})
+
+		err = deleteSkopeoPod(c, testNamespace)
+		if err != nil {
+			t.Fatalf("error deleting skopeo pod: %s", err)
+		}
+	}
+
+	// 2. setup catalog source
+	defer cleaner.NotifyTestComplete(t, true)
+
+	sourceName := genName("catalog-")
+	packageName := "busybox"
+	channelName := "alpha"
+
+	// Create gRPC CatalogSource using an external registry image and poll interval
+	var image string
+	image = testImage[9:] // strip off docker://
+	image = fmt.Sprint(image, tag)
+
+	source := &v1alpha1.CatalogSource{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       v1alpha1.CatalogSourceKind,
+			APIVersion: v1alpha1.CatalogSourceCRDAPIVersion,
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      sourceName,
+			Namespace: testNamespace,
+			Labels:    map[string]string{"olm.catalogSource": sourceName},
+		},
+		Spec: v1alpha1.CatalogSourceSpec{
+			SourceType: v1alpha1.SourceTypeGrpc,
+			Image:      image,
+			UpdateStrategy: &v1alpha1.UpdateStrategy{
+				RegistryPoll: &v1alpha1.RegistryPoll{
+					Interval: &metav1.Duration{Duration: 1 * time.Minute},
+				},
+			},
+		},
+	}
+
+	source, err = crc.OperatorsV1alpha1().CatalogSources(source.GetNamespace()).Create(source)
+	require.NoError(t, err)
+	defer func() {
+		require.NoError(t, crc.OperatorsV1alpha1().CatalogSources(source.GetNamespace()).Delete(source.GetName(), &metav1.DeleteOptions{}))
+	}()
+
+	// wait for new catalog source pod to be created
+	// Wait for a new registry pod to be created
+	selector := labels.SelectorFromSet(map[string]string{"olm.catalogSource": source.GetName()})
+	singlePod := podCount(1)
+	registryPods, err := awaitPods(t, c, source.GetNamespace(), selector.String(), singlePod)
+	require.NoError(t, err, "error awaiting registry pod")
+	require.NotNil(t, registryPods, "nil registry pods")
+	require.Equal(t, 1, len(registryPods.Items), "unexpected number of registry pods found")
+
+	// Create a Subscription for package
+	subscriptionName := genName("sub-")
+	cleanupSubscription := createSubscriptionForCatalog(t, crc, source.GetNamespace(), subscriptionName, source.GetName(), packageName, channelName, "", v1alpha1.ApprovalAutomatic)
+	defer cleanupSubscription()
+
+	// Wait for the Subscription to succeed
+	subscription, err := fetchSubscription(t, crc, testNamespace, subscriptionName, subscriptionStateAtLatestChecker)
+	require.NoError(t, err)
+	require.NotNil(t, subscription)
+
+	// Wait for csv to succeed
+	_, err = fetchCSV(t, crc, subscription.Status.CurrentCSV, subscription.GetNamespace(), csvSucceededChecker)
+	require.NoError(t, err)
+
+	registryCheckFunc := func(podList *corev1.PodList) bool {
+		if len(podList.Items) > 1 {
+			return false
+		}
+		return podList.Items[0].Status.ContainerStatuses[0].ImageID != ""
+	}
+	// get old catalog source pod
+	registryPod, err := awaitPods(t, c, source.GetNamespace(), selector.String(), registryCheckFunc)
+	// 3. Update image on registry via skopeo: this should trigger a newly updated version of the catalog source pod
+	// to be deployed after some time
+	// Make another skopeo pod to do the work of copying the image
+	if local {
+		_, err := skopeoLocalCopy(testImage, tag, catsrcImage, "new")
+		if err != nil {
+			t.Fatalf("error copying new registry file: %s", err)
+		}
+	} else {
+		skopeoArgs := skopeoCopyCmd(testImage, tag, catsrcImage, "new", registryAuth)
+		err = createSkopeoPod(c, skopeoArgs, testNamespace)
+		if err != nil {
+			t.Fatalf("error creating skopeo pod: %s", err)
+		}
+
+		// wait for skopeo pod to exit successfully
+		awaitPod(t, c, testNamespace, skopeo, func(pod *corev1.Pod) bool {
+			return pod.Status.Phase == corev1.PodSucceeded
+		})
+
+		err = deleteSkopeoPod(c, testNamespace)
+		if err != nil {
+			t.Fatalf("error deleting skopeo pod: %s", err)
+		}
+	}
+
+	// update catalog source with annotation (to kick resync)
+	source, err = crc.OperatorsV1alpha1().CatalogSources(source.GetNamespace()).Get(source.GetName(), metav1.GetOptions{})
+	require.NoError(t, err, "error awaiting registry pod")
+	source.Annotations = make(map[string]string)
+	source.Annotations["testKey"] = "testValue"
+	_, err = crc.OperatorsV1alpha1().CatalogSources(source.GetNamespace()).Update(source)
+	require.NoError(t, err, "error awaiting registry pod")
+
+	time.Sleep(11 * time.Second)
+
+	// ensure new registry pod container image is as we expect
+	podCheckFunc := func(podList *corev1.PodList) bool {
+		fmt.Printf("pod list length %d\n", len(podList.Items))
+		for _, pod := range podList.Items {
+			fmt.Printf("pod list name %v\n", pod.Name)
+		}
+
+		for _, pod := range podList.Items {
+			fmt.Printf("old image id %s\n new image id %s\n", registryPod.Items[0].Status.ContainerStatuses[0].ImageID,
+				pod.Status.ContainerStatuses[0].ImageID)
+			if pod.Status.ContainerStatuses[0].ImageID != registryPod.Items[0].Status.ContainerStatuses[0].ImageID {
+				return true
+			}
+		}
+		// update catalog source with annotation (to kick resync)
+		source, err = crc.OperatorsV1alpha1().CatalogSources(source.GetNamespace()).Get(source.GetName(), metav1.GetOptions{})
+		require.NoError(t, err, "error getting catalog source pod")
+		source.Annotations["testKey"] = genName("newValue")
+		_, err = crc.OperatorsV1alpha1().CatalogSources(source.GetNamespace()).Update(source)
+		require.NoError(t, err, "error updating catalog source pod with test annotation")
+		return false
+	}
+	// await new catalog source and ensure old one was deleted
+	registryPods, err = awaitPodsWithInterval(t, c, source.GetNamespace(), selector.String(), 30*time.Second, 10*time.Minute, podCheckFunc)
+	require.NoError(t, err, "error awaiting registry pod")
+	require.NotNil(t, registryPods, "nil registry pods")
+	require.Equal(t, 1, len(registryPods.Items), "unexpected number of registry pods found")
+
+	// update catalog source with annotation (to kick resync)
+	source, err = crc.OperatorsV1alpha1().CatalogSources(source.GetNamespace()).Get(source.GetName(), metav1.GetOptions{})
+	require.NoError(t, err, "error awaiting registry pod")
+	source.Annotations["testKey"] = "newValue"
+	_, err = crc.OperatorsV1alpha1().CatalogSources(source.GetNamespace()).Update(source)
+	require.NoError(t, err, "error awaiting registry pod")
+
+	subChecker := func(sub *v1alpha1.Subscription) bool {
+		return sub.Status.InstalledCSV == "busybox.v2.0.0"
+	}
+	// Wait for the Subscription to succeed
+	subscription, err = fetchSubscription(t, crc, testNamespace, subscriptionName, subChecker)
+	require.NoError(t, err)
+	require.NotNil(t, subscription)
+
+	// Wait for csv to succeed
+	csv, err := fetchCSV(t, crc, subscription.Status.CurrentCSV, subscription.GetNamespace(), csvSucceededChecker)
+	require.NoError(t, err)
+
+	// check version of running csv to ensure the latest version (0.9.2) was installed onto the cluster
+	v := csv.Spec.Version
+	busyboxVersion := semver.Version{
+		Major: 2,
+		Minor: 0,
+		Patch: 0,
+	}
+
+	if !reflect.DeepEqual(v, version.OperatorVersion{Version: busyboxVersion}) {
+		t.Errorf("latest version of operator not installed: catalog souce update failed")
+	}
+}
+
 func TestDependencySkipRange(t *testing.T) {
 	// Create a CatalogSource that contains the busybox v1 and busybox-dependency v1 images
 	// Create a Subscription for busybox v1, which has a dependency on busybox-dependency v1.
